package com.lanren.huhu.partner.schedule;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.lanren.huhu.partner.constants.Constants;
import com.lanren.huhu.partner.domain.AgentExpandCityAgentDetail;
import com.lanren.huhu.partner.domain.AgentMailList;
import com.lanren.huhu.partner.domain.UserAgent;
import com.lanren.huhu.partner.model.MailContentRow;
import com.lanren.huhu.partner.model.ParentAgent;
import com.lanren.huhu.partner.service.*;
import com.lanren.huhu.partner.util.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailSendException;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.mail.SendFailedException;
import java.util.*;

/**
 * @author chen
 * @title: NewAgentDailyExpandTask
 * @projectName partner
 * @description: 代理商每日邮件发送
 * @package com.lanren.huhu.partner.schedule
 * @date 2019-07-08 15:50
 */
@Component
public class AgentDailyMailTask {
    private static Logger logger = LoggerFactory.getLogger(AgentDailyMailTask.class);
    private static final String MAIL_ROW_FORMAT = "" +
            "<tr>" +
            "<td height=\"15px\">%s</td>" +
            "<td>%s</td>" +
            "<td>%s</td>" +
            "<td>%s</td>" +
            "<td>%s</td>" +
            "<td>%s</td>" +
            "</tr>";

    @Autowired
    AgentMailListService agentMailListService;
    @Autowired
    UserAgentService userAgentService;
    @Autowired
    UserService userService;
    @Autowired
    AgentExpandCityAgentDetailService agentExpandCityAgentDetailService;
    @Autowired
    MailReportService mailReportService;

    /**
     * 注入发送邮件的接口
     */
    @Autowired
    private IMailService mailService;

    private String buildMailContent(Integer agentId) {
        Date yesterday = DateUtils.getDateBefore(new Date(), 1);
        long now = System.currentTimeMillis();
        Date monthBegin = new Date(DateUtils.getMonth1stTimestamp(now));
        Date monthEnd = new Date(DateUtils.getMonthLastTimestamp(now));
        String yesterdayStr = DateUtils.format(yesterday, DateUtils.FORMAT_SHORT);
        String yesterdayBeginStr = DateUtils.format(yesterday, DateUtils.FORMAT_SHORT) + " 00:00:00";
        String yesterdayEndStr = DateUtils.format(yesterday, DateUtils.FORMAT_SHORT) + " 23:59:59";
        String monthBeginStr = DateUtils.format(monthBegin, DateUtils.FORMAT_LONG);
        String monthEndStr = DateUtils.format(monthEnd, DateUtils.FORMAT_LONG);
        StringBuilder sb = new StringBuilder();
        /**
         * 表头
         */
        MailContentRow headerRow = createRow("拓展用户的身份", "来源", "昨日新增数量(个)", "8月16日之后数量（个）", "当月总计数量（个）", "总计数量(个)");
        sb.append(formatMailRow(headerRow));
        /**
         * 代理商拓展 注册并登录用户
         */
        MailContentRow regRow = mailReportService.getRegRow(yesterdayStr, agentId, monthBeginStr, monthEndStr);
        logger.info("regRow is {}", regRow);
        sb.append(formatMailRow(regRow));
        sb.append(emptyRow());
        /**
         * 获取代理商拓展 VIP 数据
         */
        List<MailContentRow> vipRows = mailReportService.getVipRows(yesterdayBeginStr, yesterdayEndStr, agentId, monthBeginStr, monthEndStr);
        MailContentRow sumVip = createRow("Vip会员", "总计", "0", "0", "0", "0");
        MailContentRow directVip = null;
        MailContentRow indirectVip = null;
        for (MailContentRow row : vipRows) {
            if ("直接".equals(row.getC2()) && "Vip会员".equals(row.getC1())) {
                directVip = row.clone();
                logger.info("directVip is {}", directVip);
            }
            if ("间接".equals(row.getC2()) && "Vip会员".equals(row.getC1())) {
                indirectVip = row.clone();
                logger.info("indirectVip is {}", indirectVip);
            }
        }
        appendVipRows(sb, sumVip, directVip, indirectVip);
        sb.append(emptyRow());
        /**
         * 获取代理商拓展 超级VIP 数据
         */
        List<MailContentRow> sVipRows = mailReportService.getsVipRows(yesterdayBeginStr, yesterdayEndStr, agentId, monthBeginStr, monthEndStr);
        MailContentRow sumsvip = createRow("超级Vip会员", "总计", "0", "0", "0", "0");
        MailContentRow directsvip = null;
        MailContentRow indirectsvip = null;
        for (MailContentRow row : sVipRows) {
            if ("直接".equals(row.getC2()) && "超级Vip会员".equals(row.getC1())) {
                directsvip = row.clone();
                logger.info("directsvip is {}", directsvip);
            }
            if ("间接".equals(row.getC2()) && "超级Vip会员".equals(row.getC1())) {
                indirectsvip = row.clone();
                logger.info("indirectsvip is {}", indirectsvip);
            }
        }
        appendVipRows(sb, sumsvip, directsvip, indirectsvip);
        /**
         * 获取代理商拓展 层级下级代理数据
         */
        List<MailContentRow> cengjiChildRows = mailReportService.getCengjiChildRows(yesterdayBeginStr, yesterdayEndStr, agentId, monthBeginStr, monthEndStr);
        MailContentRow sumAgent = createRow("代理商", "总计", "0", "0", "0", "0");
        MailContentRow directlv2 = null;
        MailContentRow directlv34 = null;
        MailContentRow indirectlv34 = null;
        MailContentRow directlv5 = null;
        MailContentRow indirectlv5 = null;
        for (MailContentRow row : cengjiChildRows) {
            if ("直接".equals(row.getC2()) && "分公司".equals(row.getC1())) {
                directlv2 = row.clone();
                logger.info("directlv2 is {}", directlv2);
            }
            if ("直接".equals(row.getC2()) && "城市代理".equals(row.getC1())) {
                directlv34 = row.clone();
                logger.info("directlv34 is {}", directlv34);
            }
            if ("间接".equals(row.getC2()) && "城市代理".equals(row.getC1())) {
                indirectlv34 = row.clone();
                logger.info("indirectlv34 is {}", indirectlv34);
            }
            if ("直接".equals(row.getC2()) && "联盟商户".equals(row.getC1())) {
                directlv5 = row.clone();
                logger.info("directlv5 is {}", directlv5);
            }
            if ("间接".equals(row.getC2()) && "联盟商户".equals(row.getC1())) {
                indirectlv5 = row.clone();
                logger.info("indirectlv5 is {}", indirectlv5);
            }
        }
        sb.append(emptyRow());
//        /**
//         * 联盟商户
//         */
//        appendRow(sb, directlv5);
//        appendRow(sb, indirectlv5);
        /**
         * 代理商 2-4 级
         */
        sumAgentRows(sumAgent, directlv2, null);
        sumAgentRows(sumAgent, directlv34, indirectlv34);
        sb.append(formatMailRow(sumAgent));
        appendRow(sb, directlv2);
        appendRow(sb, directlv34);
        appendRow(sb, indirectlv34);
        sb.append(emptyRow());
        /**
         * 获取代理商拓展 平推下级代理数据
         */
        MailContentRow pingtuiRow = mailReportService.getPingtuiChildRow(yesterdayStr, agentId, monthBeginStr, monthEndStr);
        appendRow(sb, pingtuiRow);
        return sb.toString();
    }

    private void appendRow(StringBuilder sb, MailContentRow row) {
        if (row != null) {
            sb.append(formatMailRow(row));
        }
    }

    private void appendVipRows(StringBuilder sb, MailContentRow sumRow, MailContentRow directRow, MailContentRow indirectRow) {
        if (directRow != null) {
            sum2Row(sumRow, directRow);
        }
        if (indirectRow != null) {
            sum2Row(sumRow, indirectRow);
        }
        sb.append(formatMailRow(sumRow));
        if (directRow != null) {
            sb.append(formatMailRow(directRow));
        }
        if (indirectRow != null) {
            sb.append(formatMailRow(indirectRow));
        }
    }
    private void sumAgentRows(MailContentRow sumRow, MailContentRow directRow, MailContentRow indirectRow) {
        if (directRow != null) {
            sum2Row(sumRow, directRow);
        }
        if (indirectRow != null) {
            sum2Row(sumRow, indirectRow);
        }
    }

    private void sum2Row(MailContentRow sumRow, MailContentRow row) {
        sumRow.setC3((Integer.parseInt(row.getC3()) + Integer.parseInt(sumRow.getC3())) + "");
        sumRow.setC4((Integer.parseInt(row.getC4()) + Integer.parseInt(sumRow.getC4())) + "");
        sumRow.setC5((Integer.parseInt(row.getC5()) + Integer.parseInt(sumRow.getC5())) + "");
        sumRow.setC6((Integer.parseInt(row.getC6()) + Integer.parseInt(sumRow.getC6())) + "");
    }

    private void sendMail(AgentMailList agentMailList) {
        Date yesterday = DateUtils.getDateBefore(new Date(), 1);
        String yesterdayStr = DateUtils.format(yesterday, DateUtils.FORMAT_SHORT);
        StringBuilder sb = new StringBuilder();
        sb.append(agentMailList.getAgentName() + ", 您好:<br>");
        sb.append("<table border=1>");
        sb.append(buildMailContent(agentMailList.getAgentId()));
        sb.append("</table>");
        String[] tos = agentMailList.getAgentMail().split(";");
        String[] cc = agentMailList.getAgentCc().split(";");
        for (int i=0; i<5; i++) {
            try {
                if (StringUtils.isEmpty(cc)) {
                    mailService.sendMail(
                            "代理商每日邮件-" + yesterdayStr,
                            sb.toString(),
                            null,
                            tos
                    );
                } else {
                    mailService.sendMailWithCc(
                            "代理商每日邮件-" + yesterdayStr,
                            sb.toString(),
                            null,
                            tos,
                            cc
                    );
                }
                break;
            } catch (Exception e) {
                logger.error(e.getMessage(), e);

                if (e instanceof MailSendException) {
                    Exception[] messageExceptions = ((MailSendException) e).getMessageExceptions();
                    SendFailedException sendFail;
                    try {
                        sendFail = (SendFailedException) messageExceptions[0];
                    } catch (ClassCastException ee) {
                        logger.error("class cast exception:[{}]", ee.getMessage());
                        return;
                    }
                    logger.error("send mail eerror,the invalid mail address:{}", sendFail.getInvalidAddresses());
                }

                logger.error("发送邮件失败, 第{}次", i);
            }
        }
    }

    public void runExpandCity2ndAgentMail() {
        try {
            /**
             * 查询全部城市代理, 然后查找他们的B端推荐上级
             * 如果上级有邮件列表中的代理, 记一条拓展记录
             */
            List<AgentMailList> mailList = agentMailListService.list();
            Set<Integer> mailAgentIdSet = new HashSet<Integer>();
            for (AgentMailList agent : mailList) {
                mailAgentIdSet.add(agent.getAgentId());
            }
            List<UserAgent> cityAgentList = userAgentService.selectCityAgentList();
            List<AgentExpandCityAgentDetail> expandRecordList = new ArrayList<AgentExpandCityAgentDetail>();
            for (UserAgent cityAgent : cityAgentList) {
                try {
                    List<ParentAgent> parentAgentList = userService.getAgentListByUserId(cityAgent.getUserId(), Constants.PARENT_COLUMN_NAME_PINGTUI);
                    for (ParentAgent parentAgent : parentAgentList) {
                        try {
                            if (mailAgentIdSet.contains(parentAgent.getAgentId())) {
                                addToExpand(cityAgent, parentAgent, expandRecordList);
                            }
                        } catch (Exception e) {
                            logger.error(e.getMessage(), e);
                        }
                    }
                } catch (Exception e) {
                    logger.error(e.getMessage(), e);
                } finally {
                    Thread.sleep(200L);
                }
            }
            /**
             * 正常处理后, 写入结果
             */
            if (expandRecordList.size() > 0) {
                QueryWrapper<AgentExpandCityAgentDetail> wrapper = new QueryWrapper<AgentExpandCityAgentDetail>();
                wrapper.in("agent_id", mailAgentIdSet);
                agentExpandCityAgentDetailService.remove(wrapper);
//                logger.info("sleep");
//                Thread.sleep(15000L);
                agentExpandCityAgentDetailService.batchInsert(expandRecordList);
            }
            /**
             * 发邮件
             */
            for (AgentMailList agentMailList : mailList) {
                sendMail(agentMailList);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
    }

    private void addToExpand(UserAgent cityAgent, ParentAgent parentAgent, List<AgentExpandCityAgentDetail> expandRecordList) {
        int now = (int) (System.currentTimeMillis() / 1000L);
        AgentExpandCityAgentDetail record = new AgentExpandCityAgentDetail();
        Date expandTime = new Date(cityAgent.getActiveTime() * 1000L);
        record.setDat(expandTime);
        record.setAgentId(parentAgent.getAgentId());
        record.setUserId(parentAgent.getUserId());
        record.setExpandUserId(cityAgent.getUserId());
        record.setExpandAgentId(cityAgent.getAgentId());
        record.setExpandAgentTime(expandTime);
        record.setCreatedAt(now);
        record.setUpdatedAt(now);
        expandRecordList.add(record);
    }

    private String formatMailRow(MailContentRow mailContentRow) {
        return String.format(MAIL_ROW_FORMAT,
                mailContentRow.getC1(),
                mailContentRow.getC2(),
                mailContentRow.getC3(),
                mailContentRow.getC4(),
                mailContentRow.getC5(),
                mailContentRow.getC6()
        );
    }

    private String emptyRow() {
        return String.format(MAIL_ROW_FORMAT,
                "",
                "",
                "",
                "",
                "",
                ""
        );
    }

    private MailContentRow createRow(String c1, String c2, String c3, String c4, String c5, String c6) {
        MailContentRow row = new MailContentRow();
        row.setC1(c1);
        row.setC2(c2);
        row.setC3(c3);
        row.setC4(c4);
        row.setC5(c5);
        row.setC6(c6);
        return row;
    }
}
